// minimal upsampling algortihm by jl 2003
// Version 1.0, 2003-05-22

#include "stdafx.h"
#include <cmath>
#include <iostream>
#include <fstream>
using namespace std;

#pragma pack(1) // tell MSVC not to leave gaps in data structures
struct wavHeader
{
	// We assume that short has 16 bits and long has 32 bits.
	unsigned long riffString; // 'FFIR' (litte endian!) 
	long fileLength; // length - 8
	unsigned long waveString; // 'EVAW'
	unsigned long fmtString; // ' tmf'
	long fmtLength; // 16
	short formatTag; // 1 = PCM
	short channels; // 1, 2, ...
	long sampleRate; // samples per second, e.g. 44100
	long bytesPerSecond; // sampleRate*blockAlign
	short blockAlign; // (channels*bitsPerSample)/8
	short bitsPerSample; // 8, 16, ...
	unsigned long dataString; // 'atad'
	long dataLength;
};
#pragma pack()
// Don't forget: On big endian systems (i.e. Macs) we have to shuffle bytes!

int main(int argc, char* argv[])
{
	ifstream inFile("in.wav", ios::in | ios::binary);
	if(!inFile)
	{
		cerr << "Can't open in.wav" << endl;
		exit(-1);
	}

	wavHeader inHeader;
	inFile.read((char*) &inHeader, sizeof(inHeader));

	// code to add: check if header is filled as expected
	// code to add: check for file errors

	if(inHeader.channels != 2 || inHeader.bitsPerSample != 16)
	{
		cerr << "No standard 16 bit stereo file" << endl;
		exit(-2);
	}

	long numberOfSamples = inHeader.dataLength/4;
	short (*samples)[2] = new short[numberOfSamples][2];
	inFile.read((char*) samples, inHeader.dataLength);

	// code to add: check for file errors

	// only for upsampling,
	// downsampling has to _decimate_
	// and it has to filter _before_ decimation
	const double FACTOR = 5.343145;
	// NB: Better use only integer factors
	// and form arbitrary fractions
	// by a combination of up- and downsampling

	int newNumberOfSamples = (int)(numberOfSamples*FACTOR);
	short (*newSamples)[2] = new short[newNumberOfSamples][2];

	int i;
	for(i=0; i<newNumberOfSamples; ++i)
	{
		double t = i/FACTOR;
		int jLast = (int)t;
		int jNext = jLast+1;
		if (jNext >= numberOfSamples)
			jNext = numberOfSamples-1;

		// How far are we from the last exact sample?
		double offset = t - jLast;
		// Set offset = 0 to hear the result
		// without linear interpolation.

		int channel;
		for (channel = 0; channel < 2; ++channel)
		{
			short prev = samples[jLast][channel];
			short next = samples[jNext][channel];

			// linear mix;
			// round to next integer by taking floor(... + 0.5)
			newSamples[i][channel] = (short)
				floor((1.0-offset)*prev + offset*next + 0.5);
		}
	}

	ofstream outFile("out.wav", ios::out | ios::binary | ios::trunc);
	
	// somewhat ugly reuse of inHeader
	inHeader.dataLength = newNumberOfSamples*4;
	inHeader.fileLength = inHeader.dataLength+sizeof(wavHeader)-8;

	outFile.write((char*) &inHeader, sizeof(wavHeader));
	outFile.write((char*) newSamples, inHeader.dataLength);		

	// code to add: check for file errors

	delete[] newSamples;
	delete[] samples;

	return 0;
}